function [peaks] = mlSEDFIT_IntegrateDistribution(distribution,ranges),

dists = distribution(:,1); distc = distribution(:,2); ndist = length(dists);

figure(2); clf; hold on;
plot(dists,distc,'k-');
box on;
xlabel('sedimentation coefficient (S)');
ylabel('c(s) or ls-g*(s) (signal/S)');

% peaks.totalsignal = trapz(dists,distc);
[TotIntegral,Integral,WeightIntegral] = integratedistribution(distribution, distribution(1,1), distribution(ndist,1));
peaks.totalsignal = TotIntegral;

fprintf('\ntotal signal = %f\n',peaks.totalsignal);

if 0 == length(ranges),
    npeaks = 1;
    bintegrating = 1;
    while bintegrating,
        title(['click on peak #' num2str(npeaks) ' to be integrated (or outside the graph to stop)']);
        [x,y] = ginput(1);
        if x < min(dists) || x > max(dists),
            bintegrating = 0;
            npeaks = npeaks - 1;
            title('');
        else
            ipeak = sum(dists < x);
            cpeak = distc(ipeak);
            imin = ipeak-1;
            while imin > 0 & distc(imin) > 0.9*cpeak,
                imin = imin - 1;
                if distc(imin) > distc(imin+1), break; end;
            end;
            while imin > 1 & distc(imin-1) < distc(imin), imin = imin - 1; end;
            imax = ipeak + 1;
            while imax < ndist & distc(imax) > 0.9*cpeak,
                imax = imax + 1;
                if distc(imax) > distc(imax-1), break;end;
            end;
            while imax < ndist-1 & distc(imax+1) < distc(imax), imax = imax + 1; end;
            %             peaks.signal(npeaks) = trapz(dists(imin:imax),distc(imin:imax));
            %             peaks.sw(npeaks) = sum(distc(imin:imax).*dists(imin:imax))/sum(distc(imin:imax));
            [TotIntegral,Integral,WeightIntegral] = integratedistribution(distribution, dists(imin), dists(imax));
            peaks.signal(npeaks) = Integral;
            peaks.sw(npeaks) = WeightIntegral;
            peaks.smin(npeaks) = dists(imin);
            peaks.smax(npeaks) = dists(imax);
            patch([dists(imin); dists(imin:imax); dists(imax)],[0; distc(imin:imax); 0],'m','EdgeColor','none','FaceAlpha',0.6);
            fprintf('peak #%d (%.2fS-%.2fS): sw = %.3f, signal = %.4f (%.3f%% of total %.3f)\n',npeaks,peaks.smin(npeaks),peaks.smax(npeaks),peaks.sw(npeaks),peaks.signal(npeaks),100*peaks.signal(npeaks)/peaks.totalsignal,peaks.totalsignal);
            swtext = sprintf('s_w = %.3fS (%.2f%%)',peaks.sw(npeaks),100*peaks.signal(npeaks)/peaks.totalsignal);
            %         swtext = sprintf('s_w = %.3fS (%.2f)',peaks.sw(npeaks));
            text(peaks.sw(npeaks),max(distc(imin:imax)),swtext,'FontSize',8);
            npeaks = npeaks + 1;
        end;
    end;
else
    [npeaks, ~] = size(ranges);
    for p=1:npeaks,
        imin = sum(dists < ranges(p,1));
        imax = sum(dists <= ranges(p,2));
        %         peaks.signal(p) = trapz(dists(imin:imax),distc(imin:imax));
        %         peaks.sw(p) = sum(distc(imin:imax).*dists(imin:imax))/sum(distc(imin:imax));
        [TotIntegral,Integral,WeightIntegral] = integratedistribution(distribution, dists(imin), dists(imax));
        peaks.signal(p) = Integral;
        peaks.sw(p) = WeightIntegral;
        peaks.smin(p) = dists(imin);
        peaks.smax(p) = dists(imax);
        patch([dists(imin); dists(imin:imax); dists(imax)],[0; distc(imin:imax); 0],'m','EdgeColor','none','FaceAlpha',0.6);
        fprintf('peak #%d (%.2fS-%.2fS): sw = %.3f, signal = %.4f (%.3f%% of total %.3f)\n',npeaks,peaks.smin(p),peaks.smax(p),peaks.sw(p),peaks.signal(p),100*peaks.signal(p)/peaks.totalsignal,peaks.totalsignal);
        swtext = sprintf('s_w = %.3fS (%.2f%%)',peaks.sw(p),100*peaks.signal(p)/peaks.totalsignal);
        %         swtext = sprintf('s_w = %.3fS (%.2f)',peaks.sw(npeaks));
        text(peaks.sw(p),max(distc(imin:imax)),swtext,'FontSize',8);
    end;
end;


function [TotIntegral,cint,swint] = integratedistribution(distin, smin, smax),
dist = distin;
[nr,nc] = size(distin); if nc > nr, dist = distin'; end;
ires = length(dist);
TotIntegral = 0; Integral = 0; WeightIntegral = 0;
for i=1:ires-1,
    lefts = dist(i,1); rights = dist(i+1,1);
    dl = dist(i,2); dr = dist(i+1,2);
    if rights~=lefts,
        TotIntegral = TotIntegral + 0.5*(dl+dr)*(rights-lefts);
        slope = (dr-dl)/(rights-lefts); offs = dl-slope*lefts;
        if lefts < smin,
            if rights <= smin,
                rights = lefts;
            elseif (rights > smin) &&(rights <=smax),
                lefts = smin; 
                dl = offs + slope*smin;
            elseif rights > smax,
                lefts = smin; rights = smax;
                dl = offs + slope*smin; dr = offs + slope*smax;
            end;
        elseif (lefts >= smin)&&(lefts < smax),
            if rights <= smax,
                ;
            elseif rights > smax,
                rights = smax; dr = offs + slope*smax;
            end;
        elseif lefts > smax,
            rights = lefts;
        end;
        if lefts ~=rights,
            Integral = Integral + 0.5*(dl+dr)*(rights-lefts);
            WeightIntegral = WeightIntegral + (slope/3)*(rights*rights*rights-lefts*lefts*lefts) + 0.5*offs*(rights*rights - lefts*lefts);
        end;
    end;
    cint = Integral;
    swint = WeightIntegral/Integral;
end;

